% This program uses the method mentioned in our paper "Output Feedback ILC for a Class of Nonminimum Phase Nonlinear Systems with
%      Input Saturation: An Additive-state-decomposition-based Method" to solve the example in our paper.
% We can use different K_Gs to compare their effects. 
% The original system (24) on page 4 of our paper and our ILC structure are established in "Adjoint_Test7.slx".
% The file "Adjoint_Test7_supp.slx" is used to calculate the adjoint oprator.

clear
clc
close all

limax = 3;
limin = -15;

A = [0 1 0; 0 0 1; -4 -6 -4];
B = [0;0;1];
C = [-1 0 1];
D = 0;

%% Different feedback gains for the secondary system %%
% Notice: Choose the gain here. u1--l1, u2--l2
K_Gs = [-1 -5 -1];         % for l2 in Eq.(31)
% K_Gs = [2 5 2];              % for l1 in Eq.(31)

%% The reference trajectory %%
Ts = 0.01;
TtR = 2*pi;
Tex = 2*pi;
Tt = 6*pi;
Nt = floor(Tt/Ts+1);

u_p = zeros(Nt,2);
yd = zeros(Nt,2);
eu = zeros(Nt,2);
eu2 = zeros(Nt,2);
for i = 1:Nt
    t = (i - 1)*Ts;
    
    u_p(i,1) = t;
    u_p(i,2) = 0;
    eu(i,1) = t;
    eu(i,2) = 0;
    eu2(i,1) = t;
    eu2(i,2) = 0;
    yd(i,1) = t;
    if t < Tex
        yd(i,2) = 0;
    elseif t < Tex + TtR
        yd(i,2) = 1 - cos(t-Tex);
    else
        yd(i,2) = 0;
    end
end

%% The ILC method used in our paper %%
figure(1)
set(gcf,'color',[1 1 1])
subplot(2,1,1)
plot(u_p(:,1),yd(:,2),'g-','linewidth',3)
ylabel('y_k','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
axis([0 Tt -5.5 3.5])
title({'The convergence process of y_k and u_k'; '(The darker the line is, the larger number k is.)'},...
    'fontsize',14,'FontWeight','bold','fontname','TimesNewRoman')
legend('y_d')
hold on
subplot(2,1,2)
ylabel('u_k','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
axis([0 Tt -16 4])
hold on
plot([0 Tt],[3 3],'y--',[0 Tt],[-15 -15],'y--','linewidth',2)
legend('input saturation')
box on

figure(2)
set(gcf,'color',[1 1 1])
subplot(1,2,2)
box on
hold on
ylabel('u_k','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
axis([0 Tt -16 4])
plot([0 Tt],[3 3],'y--',[0 Tt],[-15 -15],'y--','linewidth',2)
legend('input saturation')

N2 = 20; % the number of iterations
y_p_k = cell(N2,1);
y_s_k = cell(N2,1);
y_k = cell(N2,1);
u_real_k = cell(N2,1);
u_p_k{i,1} = cell(N2,1);
u_s_k{i,1} = cell(N2,1);
u_k{i,1} = cell(N2,1);

e_max = zeros(N2,1);
for i = 1:N2
    sim('Adjoint_Test7');
    
    figure(1)
    subplot(2,1,1)
    plot(u_p(:,1),y(:,1),'color',[1-(i+5)/(N2+5),1-(i+5)/(N2+5),1-(i+5)/(N2+5)])
    subplot(2,1,2)
    plot(u_p(:,1),u_real(:,1),'color',[1-(i+5)/(N2+5),1-(i+5)/(N2+5),1-(i+5)/(N2+5)])
       
    e_max(i,1) = max(abs(e));
    y_p_k{i,1} = y_p(:,1);
    y_s_k{i,1} = y_s(:,1);
    y_k{i,1} = y(:,1);
    u_real_k{i,1} = u_real(:,1);
    u_p_k{i,1} = u_p(:,2);
    u_s_k{i,1} = u_s(:,1);
    u_k{i,1} = u_t(:,1);
    
    figure(2)
    subplot(1,2,2)
    plot(u_p(:,1),u_real(:,1),'color',[1-(2*i+N2)/(3*N2),1-(2*i+N2)/(3*N2),1-(2*i+N2)/(3*N2)])

    eu(:,2) = flipud(e_p);
    sim('Adjoint_Test7_supp');
    qf = flipud(qb);
    alpha = sum_ek(Nt,1)/sum_Gstar_ek(Nt,1);
    u_p(:,2) = u_p(:,2) + alpha*qf;  
    
    clc;
    disp(i);
end

figure(2)
subplot(1,2,1)
box on
semilogy(1:N2,e_max(:,1),'ko')
hold on
semilogy(1:N2,e_max(:,1),'b')
grid on
ylabel('|e|_m_a_x','fontsize',18,'fontname','TimesNewRoman')
xlabel('k','fontsize',18,'fontname','TimesNewRoman')
suptitle({'The convergence process of e_k and u_k'; '(In the right subpolt, the darker the line is, the larger number k is.)'})

%% Use this figure to plot the input/output of the primary/secondary system at the kth iteration %%
figure(3)
k = 2; % Notice, k can be chosen from 1 to N2
set(gcf,'color',[1 1 1])
subplot(2,1,1)
grid on
hold on
plot(u_p(:,1),yd(:,2),'g-','linewidth',3)
plot(u_p(:,1),y_p_k{k,1},'b--','linewidth',1.5)
plot(u_p(:,1),y_s_k{k,1},'r-.','linewidth',1.5)
plot(u_p(:,1),y_k{k,1},'k','linewidth',1.5)
xlim([0 Tt])
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
legend('yd','y_p_k','y_s_k','y')
title({'The input/output of the primary/secondary system'; ['when k = ', num2str(k)]},...
    'fontsize',14,'FontWeight','bold','fontname','TimesNewRoman') 
subplot(2,1,2)
grid on
hold on
plot(u_p(:,1),u_real_k{k,1},'k','linewidth',1.5)
plot(u_p(:,1),u_p_k{k,1},'b--','linewidth',1.5)
plot(u_p(:,1),u_s_k{k,1},'r-.','linewidth',1.5)
plot(u_p(:,1),u_k{k,1},'k:','linewidth',1.5)
plot([0 Tt],[3 3],'y--',[0 Tt],[-15 -15],'y--','linewidth',2)
xlim([0 Tt])
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
legend('u_k_r_e_a_l=saturation(u_k)','u_p_k','u_s_k','u_k=u_p_k+u_s_k','input saturation')