% This program reproduce the method used in paper "A pseudoinverse-based iterative learning control" %
% The original system (21) on page 834 is established in "Example.slx" %
close all
clc
clear

%% Make the system conform to system (10) on page 832 %%
A = [-1  1  0  0; ...
      0 -3  0  0; ...
      1  0 -2  0; ...
      0  0  0 -1];
B = [0.2;0;0;0];
C = [1 0 -3 0];     % the linearized system on top left of page 835
alpha = 0.001;      

Ac = [A          -B*1/alpha*B' B*1/alpha*B'; ...
      -C'*C      -A'           zeros(4,4); ...
      zeros(4,4) zeros(4,4)    -A'];
Bc = [zeros(4,1);zeros(4,1);-C'];
Cc = [zeros(1,4) -1/alpha*B' 1/alpha*B'];
nAc = size(Ac,1);

%% Make the system conform to the form mentioned in paper "Nonlinear inversion-based output tracking" %%
[V,D] = eig(Ac);
nD = length(find(D<0)); % ҳС0ĸ

PP = eye(nAc);
i = 1;
while i <= nD
    if D(i,i) > 0
        Ptemp = eye(nAc);
        Ptemp = Ptemp([1:i-1,i+1:nAc,i],:);
        D = Ptemp*D*Ptemp';
        PP = Ptemp*PP;
        i = i - 1;
    end
    i = i + 1;
end
P = PP/V;
Ad = P*Ac/P;
Bd = P*Bc;
Cd = Cc/P;
Ad(abs(Ad)<1e-10) = 0;
Bd(abs(Bd)<1e-10) = 0;
Cd(abs(Cd)<1e-10) = 0;

%% The reference trajectory %%
Ts = 0.01;
Tt = 15;
T1 = 5;
T2 = T1 + 2*pi;
Nt = Tt/Ts;

yd = zeros(Nt,2);
u = zeros(Nt,2);
e_in = zeros(Nt,2);
for i = 1:Nt+1
    t = (i-1)*Ts;
    
    yd(i,1) = t;
    if t < T1
        yd(i,2) = 0;
    elseif t <= T2
        yd(i,2) = 0.2*sin(t-T1);
    else
        yd(i,2) = 0;
    end
    
    u(i,1) = t;    
    e_in(i,1) = t;
end

%% The ILC method used in paper "A pseudoinverse-based iterative learning control" %%
sim('Example')
e_in(:,2) = e_out;

t_l = -100;
t_r = 100;
Ts_p = 0.01;

figure(1)
set(gcf,'color',[1 1 1])
subplot(2,1,1)
plot(u(:,1),yd(:,2))
grid on
hold on
ylabel('y_k','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
title({'The convergence process of y_k and u_k'; '(The darker the line is, the larger number k is.)'},...
    'fontsize',14,'FontWeight','bold','fontname','TimesNewRoman')
subplot(2,1,2)
plot(u(:,1),u(:,2))
ylabel('u_k','fontsize',18,'fontname','TimesNewRoman')
xlabel('t/s','fontsize',18,'fontname','TimesNewRoman')
grid on
hold on

N2 = 10;
for k = 1:N2
    sim('Example')
    e_in(:,2) = e_out;
    figure(1)
    subplot(2,1,1)
    plot(u(:,1),y(:,1),'color',[1-(k+5)/(N2+5),1-(k+5)/(N2+5),1-(k+5)/(N2+5)])
    subplot(2,1,2)
    plot(u(:,1),u(:,2),'color',[1-(k+5)/(N2+5),1-(k+5)/(N2+5),1-(k+5)/(N2+5)])
    
    delta_u = zeros(Nt+1,1);
    for i = 1:Nt+1
        t_t = (i-1)*Ts;
        ti_l = t_l:Ts_p:t_t;
        ti_r = t_t:Ts_p:t_r;
        n_l = length(ti_l);
        n_r = length(ti_r);
        e_in_l = zeros(n_l,2);
        e_in_r = zeros(n_r,2);
        e_in_l(:,1) = ti_l;
        e_in_r(:,1) = ti_r;
        if t_t > 0;
            e_in_l(round((0-t_l)/Ts_p+1):round((t_t-t_l)/Ts_p+1),2) = interp1(0:Ts:t_t,e_out(1:round(t_t/Ts+1)),0:Ts_p:t_t)';
        end
        if t_t < Tt
            e_in_r(1:round((Tt-t_t)/Ts_p+1),2) = interp1(t_t:Ts:Tt,e_out(round(t_t/Ts+1):round(Tt/Ts+1)),t_t:Ts_p:Tt)';
        end
        
        x_temp = zeros(size(Ad,1),1);
        for j = 1:size(Ad,1)
            if j <= nD
                temp = exp(Ad(j,j)*(t_t-ti_l))*Bd(j)*e_in_l(:,2)*Ts_p;
            else
                temp = -exp(Ad(j,j)*(t_t-ti_r))*Bd(j)*e_in_r(:,2)*Ts_p;
            end
            x_temp(j) = temp;
        end
              
        y_temp = Cd*x_temp;
        delta_u(i,1) = y_temp;
        
        clc
        disp('iterative = ')
        disp(k)
        disp('time = ')
        disp(t_t)
    end
    u(:,2) = u(:,2) + delta_u;
end

